SystemOrganization addCategory: #TusLibros!

Object subclass: #Cashier
	instanceVariableNames: 'paymentTerminal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:33'!
emptyCartCheckoutMessage
	
	^ 'Cannot checkout empty carts.'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 21:15'!
newWithPaymentTerminal: aPaymentTerminal 
	
	^ self new initializeWithPaymentTerminal: aPaymentTerminal.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:37'!
checkout: aCart usingCard: aCard
	
	self error: Cashier emptyCartCheckoutMessage! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 21:16'!
initializeWithPaymentTerminal: aPaymentTerminal
	
	paymentTerminal := aPaymentTerminal.! !

Object subclass: #PaymentTerminalStub
	instanceVariableNames: 'transactionCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 21:17'!
initialize

	transactionCount := 0.! !

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 21:16'!
transactionCount
	
	^ transactionCount! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalog products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCart class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:23'!
invalidAmountOfProductsMessage
	
	^ 'Amount of products to add should be greater than zero'.! !

!ShoppingCart class methodsFor: 'instance-creation' stamp: 'JuanEdi 5/27/2013 21:30'!
newWithCatalog: aBookCatalog 
	
	^ self new initializeWithCatalog: aBookCatalog.! !

!ShoppingCart class methodsFor: 'exception messages' stamp: 'JuanEdi 5/29/2013 20:07'!
outOfCatalogMessage
	^ 'Error adding product to cart, the product is not in the catalog.'! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 5/29/2013 20:23'!
add: aProduct withOcurrencies: aNumber 
	| currentOcurrences |
	
	self validatePossitiveAmountOfProducts: aNumber.
	self validateCatalogContainsProduct: aProduct.
		
	currentOcurrences := products at: aProduct ifAbsent: 0.
	products at: aProduct put: (currentOcurrences + aNumber).! !

!ShoppingCart methodsFor: 'initialize-release' stamp: 'JuanEdi 5/29/2013 19:49'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	products := Dictionary new.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:50'!
isEmpty

	^ products isEmpty.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:53'!
occurrencesOf: aProduct 
	
	^ products at: aProduct.! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:07'!
validateCatalogContainsProduct: aProduct

	(catalog includes: aProduct) ifFalse: [ self error: ShoppingCart outOfCatalogMessage]! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:22'!
validatePossitiveAmountOfProducts: anAmount

	(anAmount > 0) ifFalse: [ self error: ShoppingCart invalidAmountOfProductsMessage]! !

TestCase subclass: #ShoppingCartTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCartTest commentStamp: '<historical>' prior: 0!
A ShoppingCartTest is a test class for testing the behavior of ShoppingCart!

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/27/2013 21:38'!
testCartShouldBeEmptyAfterCreation

	self assert: (ShoppingCart newWithCatalog: #('123')) isEmpty.! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 19:56'!
testShouldContainBookAfterAddingItToACart
	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	cart add: '123' withOcurrencies: 2.
	self assert: (cart occurrencesOf: '123') = 2.
	! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 20:24'!
testShouldFailIfTryingToAddANegativeAmountOfProducts

	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	cart add: '123' withOcurrencies: 2.
	self should: [ cart add: '123' withOcurrencies: -1 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart invalidAmountOfProductsMessage ].
	self assert: (cart occurrencesOf: '123') = 2.
	! !

!ShoppingCartTest methodsFor: '2-Checkout' stamp: 'JuanEdi 5/29/2013 21:13'!
testShouldFailIfTryingToCheckoutEmptyCart
	
	| cashier paymentTerminal initialTransactionCount |
	
	paymentTerminal := PaymentTerminalStub new.
	cashier := Cashier newWithPaymentTerminal: paymentTerminal.
	
	initialTransactionCount := paymentTerminal transactionCount.
	
	self should: [ cashier checkout: (ShoppingCart newWithCatalog: #()) usingCard: '12345' ]
		  raise: Error
		  withExceptionDo: [ :anException | self assert: anException messageText = Cashier emptyCartCheckoutMessage ].
	
	self assert: paymentTerminal transactionCount = initialTransactionCount.! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 20:07'!
testShouldOnlyAcceptExistingBooks
	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	self should: [ cart add: 'InexistentBook' withOcurrencies: 2 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart outOfCatalogMessage ].
	self assert: cart isEmpty.
	! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 5/29/2013 19:57'!
testShouldUpdateBookOrderCountAfterAddingMoreCopies
	| cart |
	cart := ShoppingCart newWithCatalog: #('123').
	cart add: '123' withOcurrencies: 2 .
	cart add: '123' withOcurrencies: 3.
	self assert: (cart occurrencesOf: '123') = 5.! !
